Attribute VB_Name = "Helper"
' Copyright (c) Traeger Industry Components GmbH.  All Rights Reserved.
Option Explicit


Private m_addressFactory As PlcAddressFactory
Private m_addressHelper As PlcAddressHelper
Private m_arrayFactory As PlcArrayFactory
Private m_deviceFactory As SimaticDeviceFactory
Private m_endpointFactory As IPDeviceEndPointFactory
Private m_operandHelper As PlcOperandHelper
Private m_valueFactory As PlcValueFactory


Public Property Get PlcAddresses() As PlcAddressHelper
    If (m_addressHelper Is Nothing) Then
        Set m_addressHelper = New PlcAddressHelper
    End If

    Set PlcAddresses = m_addressHelper
End Property


Public Property Get PlcOperands() As PlcOperandHelper
    If (m_operandHelper Is Nothing) Then
        Set m_operandHelper = New PlcOperandHelper
    End If

    Set PlcOperands = m_operandHelper
End Property


Public Function PlcOperand_DataBlock(ByVal number As Long) As PlcOperand
    If (m_operandHelper Is Nothing) Then
        Set m_operandHelper = New PlcOperandHelper
    End If
    
    Set PlcOperand_DataBlock = m_operandHelper.DataBlock(number)
End Function


Public Function PlcOperand_Of(ByVal operandType As PlcOperandType) As PlcOperand
    If (m_operandHelper Is Nothing) Then
        Set m_operandHelper = New PlcOperandHelper
    End If
    
    Set PlcOperand_Of = m_operandHelper.Of(operandType)
End Function


Public Function New_SimaticDevice( _
        ByRef endPoint As PlcDeviceEndPoint, _
        Optional ByVal deviceType As SimaticDeviceType = SimaticDeviceType_S7300_400) As SimaticDevice

    If (m_deviceFactory Is Nothing) Then
        Set m_deviceFactory = New SimaticDeviceFactory
    End If
        
    Set New_SimaticDevice = m_deviceFactory.Create(endPoint, deviceType)
End Function


Public Function New_SimaticIPDevice( _
        ByRef address As String, _
        Optional ByVal rack As Long = 0, _
        Optional ByVal slot As Long = 2, _
        Optional ByVal deviceType As SimaticDeviceType = SimaticDeviceType_S7300_400) As SimaticDevice

    Set New_SimaticIPDevice = New_SimaticDevice( _
            New_IPDeviceEndPoint(address, rack, slot), deviceType)
End Function


Public Function New_IPDeviceEndPoint( _
        ByRef address As String, _
        Optional ByVal rack As Long = 0, _
        Optional ByVal slot As Long = 2) As IPDeviceEndPoint
        
    If (m_addressFactory Is Nothing) Then
        Set m_endpointFactory = New IPDeviceEndPointFactory
    End If
        
    Set New_IPDeviceEndPoint = m_endpointFactory.Create(address, rack, slot)
End Function


Public Function New_PlcAddress( _
        ByRef operand As PlcOperand, _
        ByVal rawType As PlcRawType, _
        ByVal byteNumber As Long, _
        Optional bitNumber As Long = -1)

    If (m_addressFactory Is Nothing) Then
        Set m_addressFactory = New PlcAddressFactory
    End If

    Set New_PlcAddress = m_addressFactory.Create(operand, rawType, byteNumber, bitNumber)
End Function


Public Function New_PlcBoolean( _
        ByRef address As String, _
        Optional ByVal value As Boolean = False) As PlcBoolean
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcBoolean = m_valueFactory.CreateBoolean(address, value)
End Function


Public Function New_PlcBooleanArray( _
        ByRef address As String, _
        ByVal length As Long) As PlcBooleanArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcBooleanArray = m_arrayFactory.CreateBooleanArray(address, length)
End Function


Public Function New_PlcBooleanArrayUsingValues( _
        ByRef address As String, _
        ParamArray booleanValues() As Variant) As PlcBooleanArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As Boolean
    ReDim values(UBound(booleanValues))
    
    Dim index As Long
    For index = 0 To UBound(booleanValues)
        values(index) = CBool(booleanValues(index))
    Next
    
    Set New_PlcBooleanArrayUsingValues = m_arrayFactory.CreateBooleanArrayUsingValues(address, values)
End Function


Public Function New_PlcByte( _
        ByRef address As String, _
        Optional ByVal value As Byte = 0) As PlcByte
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcByte = m_valueFactory.CreateByte(address, value)
End Function


Public Function New_PlcByteArray( _
        ByRef address As String, _
        ByVal length As Long) As PlcByteArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcByteArray = m_arrayFactory.CreateByteArray(address, length)
End Function


Public Function New_PlcByteArrayUsingValues( _
        ByRef address As String, _
        ParamArray byteValues() As Variant) As PlcByteArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As Byte
    ReDim values(UBound(byteValues))
    
    Dim index As Long
    For index = 0 To UBound(byteValues)
        values(index) = CByte(byteValues(index))
    Next
    
    Set New_PlcByteArrayUsingValues = m_arrayFactory.CreateByteArrayUsingValues(address, values)
End Function


Public Function New_PlcChar( _
        ByRef address As String, _
        Optional ByRef value As String = "") As PlcChar
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcChar = m_valueFactory.CreateChar(address, value)
End Function


Public Function New_PlcCharArray( _
        ByRef address As String, _
        ByVal length As Long) As PlcCharArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcCharArray = m_arrayFactory.CreateCharArray(address, length)
End Function


Public Function New_PlcCharArrayUsingValues( _
        ByRef address As String, _
        ParamArray stringValues() As Variant) As PlcCharArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As String
    ReDim values(UBound(stringValues))
    
    Dim index As Long
    For index = 0 To UBound(stringValues)
        values(index) = CStr(stringValues(index))
    Next
    
    Set New_PlcCharArrayUsingValues = m_arrayFactory.CreateCharArrayUsingValues(address, values)
End Function


Public Function New_PlcDate( _
        ByRef address As String, _
        Optional ByRef value As Date) As PlcDate
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcDate = m_valueFactory.CreateDate(address, value)
End Function


Public Function New_PlcDateTime( _
        ByRef address As String, _
        Optional ByRef value As Date) As PlcDateTime
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcDateTime = m_valueFactory.CreateDateTime(address, value)
End Function


Public Function New_PlcDouble( _
        ByRef address As String, _
        Optional ByVal value As Double = 0) As PlcDouble
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcDouble = m_valueFactory.CreateDouble(address, value)
End Function


Public Function New_PlcDoubleArray( _
        ByRef address As String, _
        ByVal length As Long) As PlcDoubleArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcDoubleArray = m_arrayFactory.CreateDoubleArray(address, length)
End Function


Public Function New_PlcDoubleArrayUsingValues( _
        ByRef address As String, _
        ParamArray doubleValues() As Variant) As PlcDoubleArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As Double
    ReDim values(UBound(doubleValues))
    
    Dim index As Long
    For index = 0 To UBound(doubleValues)
        values(index) = CDbl(doubleValues(index))
    Next
    
    Set New_PlcDoubleArrayUsingValues = m_arrayFactory.CreateDoubleArrayUsingValues(address, values)
End Function


Public Function New_PlcInt16( _
        ByRef address As String, _
        Optional ByVal value As Integer = 0) As PlcInt16
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcInt16 = m_valueFactory.CreateInt16(address, value)
End Function


Public Function New_PlcInt16Array( _
        ByRef address As String, _
        ByVal length As Long) As PlcInt16Array
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcInt16Array = m_arrayFactory.CreateInt16Array(address, length)
End Function


Public Function New_PlcInt16ArrayUsingValues( _
        ByRef address As String, _
        ParamArray integerValues() As Variant) As PlcInt16Array
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As Integer
    ReDim values(UBound(integerValues))
    
    Dim index As Long
    For index = 0 To UBound(integerValues)
        values(index) = CInt(integerValues(index))
    Next
    
    Set New_PlcInt16ArrayUsingValues = m_arrayFactory.CreateInt16ArrayUsingValues(address, values)
End Function


Public Function New_PlcInt32( _
        ByRef address As String, _
        Optional ByVal value As Long = 0) As PlcInt32
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcInt32 = m_valueFactory.CreateInt32(address, value)
End Function


Public Function New_PlcInt32Array( _
        ByRef address As String, _
        ByVal length As Long) As PlcInt32Array
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcInt32Array = m_arrayFactory.CreateInt32Array(address, length)
End Function


Public Function New_PlcInt32ArrayUsingValues( _
        ByRef address As String, _
        ParamArray longValues() As Variant) As PlcInt32Array
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As Long
    ReDim values(UBound(longValues))
    
    Dim index As Long
    For index = 0 To UBound(longValues)
        values(index) = CLng(longValues(index))
    Next
    
    Set New_PlcInt32ArrayUsingValues = m_arrayFactory.CreateInt32ArrayUsingValues(address, values)
End Function


Public Function New_PlcReal( _
        ByRef address As String, _
        Optional ByVal value As Single = 0) As PlcReal
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcReal = m_valueFactory.CreateReal(address, value)
End Function


Public Function New_PlcRealArray( _
        ByRef address As String, _
        ByVal length As Long) As PlcRealArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Set New_PlcRealArray = m_arrayFactory.CreateRealArray(address, length)
End Function


Public Function New_PlcRealArrayUsingValues( _
        ByRef address As String, _
        ParamArray singleValues() As Variant) As PlcRealArray
    
    If (m_arrayFactory Is Nothing) Then
        Set m_arrayFactory = New PlcArrayFactory
    End If
    
    Dim values() As Single
    ReDim values(UBound(singleValues))
    
    Dim index As Long
    For index = 0 To UBound(singleValues)
        values(index) = CSng(singleValues(index))
    Next
    
    Set New_PlcRealArrayUsingValues = m_arrayFactory.CreateRealArrayUsingValues(address, values)
End Function


Public Function New_PlcS5Time( _
        ByRef address As String, _
        Optional ByRef value As Date) As PlcS5Time
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcS5Time = m_valueFactory.CreateS5Time(address, value)
End Function


Public Function New_PlcString( _
        ByRef address As String, _
        Optional ByRef value As String = "", _
        Optional ByVal length As Long = 0) As PlcString
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    If (length = 0) Then
        length = Len(value)
    End If
    
    Set New_PlcString = m_valueFactory.CreateString(address, value, length)
End Function


Public Function New_PlcTime( _
        ByRef address As String, _
        Optional ByRef value As Date) As PlcTime
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcTime = m_valueFactory.CreateTime(address, value)
End Function


Public Function New_PlcTimeOfDay( _
        ByRef address As String, _
        Optional ByRef value As Date) As PlcTimeOfDay
    
    If (m_valueFactory Is Nothing) Then
        Set m_valueFactory = New PlcValueFactory
    End If
    
    Set New_PlcTimeOfDay = m_valueFactory.CreateTimeOfDay(address, value)
End Function


Public Function New_PlcValues( _
        ParamArray values() As Variant) As PlcValue()

    Dim plcValues() As PlcValue
    ReDim plcValues(UBound(values))
    
    Dim index As Long
    For index = 0 To UBound(values)
        Set plcValues(index) = values(index)
    Next
    
    New_PlcValues = plcValues
End Function
        

Public Function New_PlcValuesCollection( _
        ParamArray values() As Variant) As PlcValueCollection

    Dim plcValues As PlcValueCollection
    Set plcValues = New PlcValueCollection
    
    Dim index As Long
    For index = 0 To UBound(values)
        Call plcValues.Add(values(index))
    Next
    
    Set New_PlcValuesCollection = plcValues
End Function

